'' 
'' このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
'' © MESCIUS inc. All rights reserved.
'' 
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Threading
Imports System.IO
Imports System.Linq
Imports System.Drawing
Imports System.Runtime.CompilerServices
Imports System.Runtime.InteropServices
Imports GrapeCity.Documents.Drawing
Imports GrapeCity.Documents.Text
Imports GrapeCity.Documents.Pdf
Imports GrapeCity.Documents.Pdf.Annotations
Imports GrapeCity.Documents.Pdf.Actions
Imports GrapeCity.Documents.Html
Imports GCTEXT = GrapeCity.Documents.Text
Imports GCDRAW = GrapeCity.Documents.Drawing

Public Module Util

    Public Const LargeDocumentIterations = 1000
    Private s_gen As Integer = 0
    Private s_fixedTime As DateTime = New DateTime(1989, 6, 21, 1, 23, 45, DateTimeKind.Utc)

    Public Property NoRandom As Boolean = False

    Public Function NewRandom() As Random
        If NoRandom Then
            Return New Random(0)
        Else
        Return New Random(Convert.ToInt32(DateTime.Now.Ticks And Int32.MaxValue) + Interlocked.Increment(s_gen))
        End If
    End Function

    Public Function TimeNow() As DateTime
        If NoRandom Then
            Return s_fixedTime
        Else
            Return DateTime.Now
        End If
    End Function

    Private s_words As String() =
        {
            "lorem", "ipsum", "dolor", "sit", "amet", "consectetur", "adipiscing", "elit", "mauris", "id", "volutpat",
            "tellus", "ullamcorper", "sem", "praesent", "et", "ante", "laoreet", "lobortis", "nunc", "congue",
            "nisi", "donec", "ac", "tempus", "sed", "feugiat", "pulvinar", "pharetra", "turpis", "nonummy",
            "at", "felis", "eget", "molestie", "euismod", "non", "aliquet", "diam", "proin", "mi", "nibh",
            "massa", "tincidunt", "ut", "dolore", "magna", "aliquam", "erat"
        }

    Private s_browserBasePath As String = Nothing
    Private ReadOnly s_browserBasePath_lock As New Object()

    '' 既知の場所からブラウザを使用する場合に設定します。
    Public Sub SetBrowserBasePath(ByVal browserBasePath As String)
        SyncLock s_browserBasePath_lock
            s_browserBasePath = browserBasePath
        End SyncLock
    End Sub

    '' GcHtml V6J以降：このメソッドは、オンラインサンプルだけでなく、ユーザーがダウンロードした
    '' すべてのサンプルでも使用できるひとつのターゲットロケーションを提供するため、
    '' 各サンプルが独自にブラウザをダウンロードしてインストールするのではなく、
    '' 複数のサンプルが同じブラウザのインストールを共有できます。
    '' Chrome がインストールされているシステムでは、BrowserFetcher.GetSystemChromePath() を呼び出すと、
    '' インストールされている Chrome ブラウザが使用されます。
    Public Function GetBrowserPath() As String
        If s_browserBasePath Is Nothing Then
            Dim systemPath = BrowserFetcher.GetSystemChromePath()
            If systemPath IsNot Nothing Then
                Return systemPath
            End If
        End If
        Dim pth As String
        If s_browserBasePath Is Nothing Then
            pth = Path.Combine(Path.GetTempPath(), ".gc-chromium")
        Else
            pth = s_browserBasePath
        End If
        Dim bf = New BrowserFetcher() With {.DestinationFolder = pth}
        Return bf.GetDownloadedPath()
    End Function

    '' GcHtml V6J:
    Public Function NewHtmlBrowser() As GcHtmlBrowser
        Dim pth = GetBrowserPath()
        Try
            If (RuntimeInformation.IsOSPlatform(OSPlatform.Linux)) Then
                Return New GcHtmlBrowser(pth, New LaunchOptions With {.RunWithNoSandbox = True})
            Else
                Return New GcHtmlBrowser(pth)
            End If
        Catch ex As Exception
            If pth = BrowserFetcher.GetSystemChromePath() Then
                Throw
            Else
                Throw New Exception("Exception occurred when launching the browser (see inner exception for details), " +
                    $"the browser installation in {Path.GetDirectoryName(pth)} may be corrupt. Please remove it and try again.", ex)
            End If
        End Try
    End Function

    '' https://stackoverflow.com/questions/4286487/is-there-any-lorem-ipsum-generator-in-c
    Public Function LoremIpsum(
            Optional ByVal numParagraphs As Integer = 5,
            Optional ByVal minSentences As Integer = 5,
            Optional ByVal maxSentences As Integer = 10,
            Optional ByVal minWords As Integer = 5,
            Optional ByVal maxWords As Integer = 30) As String

        Dim commaMax = 10
        Dim rnd = NewRandom()
        Dim numSentences = rnd.Next(maxSentences - minSentences) + minSentences + 1
        Dim numWords = rnd.Next(maxWords - minWords) + minWords + 1

        Dim result = New StringBuilder()
        For p = 1 To numParagraphs
            For s = 1 To numSentences
                For w = 1 To numWords
                    Dim word = s_words(rnd.Next(s_words.Length))
                    If w = 1 Then
                        result.Append(Char.ToUpper(word(0)))
                        result.Append(word.Substring(1))
                    Else
                        If rnd.Next(commaMax) = 0 Then
                            result.Append(",")
                        End If
                        result.Append(" ")
                        result.Append(word)
                    End If
                Next
                result.Append(". ")
            Next
            result.Append(vbCrLf)
        Next
        Return result.ToString()
    End Function

    '' Gets the list of words used by the LoremIpsum method.
    Public Function LoremWords() As List(Of String)
        Return s_words.ToList()
    End Function

    Public Function getString_ja(
            Optional ByVal numFullSentence As Integer = 1,
            Optional ByVal numSentence As Integer = 0,
            Optional ByVal numParagraphs As Integer = 5,
            Optional ByVal minSentences As Integer = 2,
            Optional ByVal maxSentences As Integer = 30,
            Optional ByVal sentenceset As Boolean = True) As String

        '' サンプル文字列  30件
        Dim s_wordsJP =
            {
            "うみ", "やま", "そら", "にじ", "くも",
            "Pen", "Desk", "Book", "Chair", "Text",
            "壱弐参四五", "67890", "平成30年10月17日", "2018/10/17", "〒981-3205",
            "!", "?", "(", ")", "+",
            "！", "？", "【", "】", " ※",
            "鯵", "鯖", "鮭", "鰯", "鮪", "鮑", "鮃", "鰈", "鯛", "鰹",
            "ウニ", "イクラ", "アナゴ", "エビ", "タコ", "アサリ", "イカ", "アユ", "ナマズ", "サメ"
            }

        '' 宮沢賢治 「雨ニモマケズ（現代文語表記）」 30件
        Dim s_amenimo =
        {
            "雨にも負けず", "風にも負けず", "雪にも夏の暑さにも負けぬ", "丈夫なからだをもち", "慾はなく",
            "決して怒らず", "いつも静かに笑っている", "一日に玄米四合と", "味噌と少しの野菜を食べ", "あらゆることを",
            "自分を勘定に入れずに", "よく見聞きし分かり", "そして忘れず", "野原の松の林の陰の", "小さな萱ぶきの小屋にいて",
            "東に病気の子供あれば", "行って看病してやり", "西に疲れた母あれば", "行ってその稲の束を負い", "南に死にそうな人あれば",
            "行ってこわがらなくてもいいといい", "北に喧嘩や訴訟があれば", "つまらないからやめろといい", "日照りの時は涙を流し", "寒さの夏はおろおろ歩き",
            "みんなにでくのぼーと呼ばれ", "褒められもせず", "苦にもされず", "そういうものに", "わたしはなりたい"
        }

        Dim rand = NewRandom()

        Dim result As StringBuilder = New StringBuilder()
        Dim wordset As List(Of String) = s_amenimo.ToList()
        Dim spChar As String = "、"

        If sentenceset = False Then
            wordset = s_wordsJP.ToList()
            spChar = " "
        End If

        Dim wdslen As Integer = wordset.ToArray().Length
        maxSentences = If((maxSentences > wdslen), wdslen, maxSentences)

        For p As Integer = 0 To numParagraphs - 1

            Dim numSentences As Integer = wdslen
            If p > numFullSentence - 1 Then
                numSentences = rand.[Next](minSentences, maxSentences)
            End If

            For s As Integer = 0 To numSentences - 1
                Dim word As String

                If sentenceset = False Then
                    word = wordset(rand.[Next](wdslen))
                Else
                    word = wordset(s)
                End If

                If s > 0 Then

                    If numSentence = 0 Then
                        result.Append(spChar)
                    Else
                        result.Append(vbLf)
                    End If
                End If

                result.Append(word)
            Next

            result.Append("。" & vbLf)
            result.Append(vbLf)
        Next

        result.Append(vbCrLf)
        Return result.ToString()

    End Function


    '' https://stackoverflow.com/questions/273313/randomize-a-listt
    <Extension()>
    Public Sub Shuffle(Of t)(ByVal list As IList(Of t))
        Dim rnd = NewRandom()
        Dim n = list.Count
        While (n > 1)
            n -= 1
            Dim k = rnd.Next(n + 1)
            Dim value = list(k)
            list(k) = list(n)
            list(n) = value
        End While
    End Sub

    ''' <summary>
    ''' ドキュメントにメモを追加します。色のついた背景の上にテキストが表示されます。
    ''' サンプルドキュメントに情報を挿入するために使用します。
    ''' </summary>
    ''' <param name="text">描画するテキスト</param>
    ''' <param name="page">描画するページ</param>
    ''' <param name="bounds">テキストの境界。NULLの場合、1インチの余白があるページ全体が使用されます。
    ''' NULLではなく高さが0の場合、余白を指定するために使用されます。</param>
    ''' <returns>メモが占める実際の矩形</returns>
    Public Function AddNote(ByVal text As String, ByVal page As Page, Optional ByVal bounds As RectangleF? = Nothing) As RectangleF
        Dim g = page.Graphics
        Dim tl = g.CreateTextLayout()
        Dim pad = g.Resolution / 8.0F
        tl.DefaultFormat.Font = StandardFonts.Helvetica
        Dim yumin = Path.Combine("Resources", "Fonts", "ipag.ttc")
        If (File.Exists(yumin)) Then
            tl.DefaultFormat.Font.AddLinkedFont(GCTEXT.Font.FromFile(yumin))
        End If
        tl.DefaultFormat.FontSize = 12
        If Not bounds.HasValue Then
            Dim margin = g.Resolution ' 1" margins by default
            bounds = New RectangleF(margin, margin, page.Size.Width - margin * 2, page.Size.Height - margin * 2)
        ElseIf bounds.Value.Height = 0 Then
            bounds = New RectangleF(bounds.Value.Location, New SizeF(bounds.Value.Width, page.Size.Height - bounds.Value.Top * 2))
        End If
        tl.MaxWidth = bounds.Value.Width
        tl.MaxHeight = bounds.Value.Height
        tl.MarginAll = pad

        Dim texts = Regex.Split(text, "(https?://[^ ]+)", RegexOptions.IgnoreCase)
        Dim runs = New List(Of TextRun)()
        Dim urls = New List(Of String)()

        For Each txt In texts
            If (txt.StartsWith("http", StringComparison.OrdinalIgnoreCase)) Then
                urls.Add(txt)
                runs.Add(tl.Append(txt, New TextFormat(tl.DefaultFormat) With {.ForeColor = Color.Blue, .Underline = True, .TextRunAsCluster = True}))
            Else
                tl.Append(txt)
            End If
        Next
        tl.PerformLayout(True)
        For i = 0 To runs.Count - 1
            Dim run = runs(i)
            Dim rects = tl.GetTextRects(run.CodePointIndex, run.CodePointCount)
            For Each r In rects
                Dim rc = r.ToRectangleF()
                rc.Offset(bounds.Value.Location)
                page.Annotations.Add(New LinkAnnotation(rc, New ActionURI(urls(i))))
            Next
        Next

        Dim rect = tl.ContentRectangle
        rect.Offset(bounds.Value.Location)
        rect.Inflate(pad, pad)

        g.FillRectangle(rect, Color.FromArgb(213, 240, 221))
        g.DrawRectangle(rect, Color.FromArgb(59, 170, 92), 0.5F)
        g.DrawTextLayout(tl, bounds.Value.Location)

        Return rect
    End Function

    ''' <summary>
    ''' ディスクファイルから画像を読み込みます
    ''' <para>
    ''' このメソッドは、Windows Imaging Component フレームワークを介して動作する
    ''' GrapeCity.Documents.Drawing.Windows.WicImage を使用することで、
    ''' Windows システム上でより良いパフォーマンスを提供します。
    ''' WicImage がサポートされていない場合、このメソッドはデフォルトで
    ''' プラットフォーム非依存の Image.FromFile() メソッドになります。
    ''' </para>
    ''' </summary>
    ''' <param name="path">画像のパス</param>
    ''' <returns>画像オブジェクト</returns>
    Public Function ImageFromFile(ByVal path As String) As IImage
        If (GrapeCity.Documents.Imaging.Windows.GcWicBitmap.IsSupported) Then
            Return New GrapeCity.Documents.Imaging.Windows.GcWicBitmap(path)
        Else
            Return GCDRAW.Image.FromFile(path)
        End If
    End Function

    Public Function getFont() As GCTEXT.Font
        Dim return_font As GCTEXT.Font = StandardFonts.Times

        Try
            return_font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "ipag.ttc"))
        Catch ex As Exception
            Console.WriteLine(ex.Message.ToString())
            Console.WriteLine("フォントを " & return_font.FontFamilyName.ToString() & " に設定します")
        End Try

        Return return_font
    End Function
End Module
