'' 
'' このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
'' © MESCIUS inc. All rights reserved.
'' 
Imports System
Imports System.IO
Imports System.Reflection
Imports System.Collections.Generic
Imports System.Diagnostics
Imports GrapeCity.Documents.Pdf
Imports GrapeCity.Documents.Pdf.Layers
''
'' このプログラムは、Hyphenation クラスのインスタンスを作成し、それを使用して PDF ファイルを生成します。
'' また、生成された PDF をマルチフレームの TIFF として保存し、PDF の最初のページを SVG として保存します
'' （他に、JPEG、PNG、BMP、GIF 形式への保存にも対応しています）。
'' このサンプルのビルドと実行に必要な DioDocs パッケージは NuGet.org から入手可能です。
''
'' コマンドラインスイッチ：
'' /y, -y ：すべてデフォルトとみなし、既存のファイルを上書きします。
'' /q, -q ：エコーをオフにします（/y の指定がない場合は無視されます）。
'' /n, -n ：反復可能な「random」データを生成します。
'' /f path, -f path ：ベースファイルのパス/名前として 'name' を使用します。
''
Module Program
    Sub Main(args As String())
        Dim yes2all = False
        Dim quiet = False
        Dim norandom = False
        Dim outMs As MemoryStream = Nothing
        Dim outTw As TextWriter = Nothing
        Try
            Dim fname = "Hyphenation"
            For i = 0 to args.Length - 1
                Select Case args(i).ToLowerInvariant()
                    Case "/y", "-y"
                        yes2all = True
                    Case "/q", "-q"
                        quiet = True
                    Case "/n", "-n"
                        norandom = True
                    Case "/f", "-f"
                        If i + 1 < args.Length Then
                            i += 1
                            If Directory.Exists(args(i)) Then
                                fname = Path.Combine(args(i), fname)
                            Else
                                fname = args(i)
                            End If
                        End If
                End Select
            Next i
            If norandom Then
                Util.NoRandom = True
            End If
            If Not yes2all Then
                quiet = False
            End If
            If quiet Then
                outMs = New MemoryStream()
                outTw = New StreamWriter(outMs)
                Console.SetOut(outTw)
            End If

            Dim lk = Environment.GetEnvironmentVariable("DOCSOL_LKEY")
            if Not String.IsNullOrEmpty(lk) Then
                GcPdfDocument.SetLicenseKey(lk)
            End If

            Dim pdfName = Path.ChangeExtension(fname, ".pdf")
            Dim tiffName = Path.ChangeExtension(fname, ".tiff")
            Dim svgName = Path.ChangeExtension(fname, ".svg")

            If Not yes2all Then
                Console.WriteLine($"ENTER を押下して '{pdfName}' を作成するか、別のファイル名を入力してください：")
                Dim t = Console.ReadLine()
                If Not String.IsNullOrWhiteSpace(t) Then
                    pdfName = t
                End If
                pdfName = Path.GetFullPath(pdfName)
                If (File.Exists(pdfName)) Then
                    Console.WriteLine($"'{pdfName}' は既に存在します。{vbLf}上書きしますか？(Y/n)")
                    Dim yesno = Console.ReadLine()
                    If yesno.ToLower() = "n" Then
                        Console.WriteLine("中断しました。")
                        Return
                    End If
                End If
            End If
            If Not quiet Then
                Console.WriteLine($"'{pdfName}' を生成しています...")
            End If
            Dim sample = New Hyphenation()

            Using fs = New FileStream(pdfName, FileMode.Create)
                sample.CreatePDF(fs)
                If Not quiet Then
                    Console.WriteLine($"'{pdfName}' は正常に作成されました。")
                End If
                If Not yes2all Then
                    '' TIFF のファイル名を取得します：
                    Console.WriteLine($"Enter キーを押下して '{tiffName}' を作成するか、別のファイル名を入力してください：")
                    Dim t = Console.ReadLine()
                    If Not String.IsNullOrWhiteSpace(t) Then
                        tiffName = t
                    End If
                    tiffName = Path.GetFullPath(tiffName)
                    If (File.Exists(tiffName)) Then
                        Console.WriteLine($"'{tiffName}' は既に存在します。{vbLf}上書きしますか？(Y/n)")
                        Dim yesno = Console.ReadLine()
                        If yesno.ToLower() = "n" Then
                            Console.WriteLine("中断しました。")
                            Return
                        End If
                    End If
                    '' SVG のファイル名を取得します：
                    Console.WriteLine($"Enter キーを押下して '{svgName}' を作成するか、別のファイル名を入力してください：")
                    t = Console.ReadLine()
                    If Not String.IsNullOrWhiteSpace(t) Then
                        svgName = t
                    End If
                    svgName = Path.GetFullPath(svgName)
                    If (File.Exists(svgName)) Then
                        Console.WriteLine($"'{svgName}' は既に存在します。{vbLf}上書きしますか？(Y/n)")
                        Dim yesno = Console.ReadLine()
                        If yesno.ToLower() = "n" Then
                            Console.WriteLine("中断しました。")
                            Return
                        End If
                    End If
                End If
                '' TIFF として保存します：
                If Not quiet Then
                    Console.WriteLine($"'{tiffName}' を生成しています...")
                End If
                fs.Seek(0, SeekOrigin.Begin)
                Dim pdf = New GcPdfDocument()
                '' 注：GcPdf のライセンスがない場合、読み込まれるのは最大５ページまでです。
                pdf.Load(fs)
                '' エクスポート時にデフォルトのレイヤー構成が使用されるように、デフォルトのビュー状態を作成します：
                Dim vs = New ViewState(pdf)
                '' ドキュメントページは、サポートされている他の画像形式でも保存できます。
                '' 詳細については、GcPdfDocument.SaveAs<format> または Page.SaveAs<format> を参照してください。
                pdf.SaveAsTiff(tiffName, vs)
                If Not quiet Then
                    Console.WriteLine($"'{tiffName}' は正常に作成されました。")
                    Console.WriteLine($"'{svgName}' を生成しています...")
                End If
                '' SVG として保存します：
                pdf.Pages(0).SaveAsSvg(svgName, vs)
                If Not quiet Then
                    Console.WriteLine($"'{svgName}' は正常に作成されました。")
                End If
                '' ドキュメントのページは、サポートされている他の画像形式に保存することもできます。
                '' 詳細については、GcPdfDocument.SaveAs<形式> または Page.SaveAs<形式> を参照してください。
            End Using

            '' 生成された PDF ファイルを開くには、以下のコメントを解除してください：
            '' Dim psi = new ProcessStartInfo() With { .FileName = pdfName, .UseShellExecute = true }
            '' Process.Start(psi)

            '' 生成された TIFF ファイルを開くには、以下のコメントを解除してください：
            '' Dim psiTiff = new ProcessStartInfo() With { .FileName = tiffName, .UseShellExecute = true }
            '' Process.Start(psiTiff)

            '' 生成された SVG ファイルを開くには、以下のコメントを解除してください：
            '' Dim psiSvg = new ProcessStartInfo() With { .FileName = svgName, .UseShellExecute = true }
            '' Process.Start(psiSvg)
        Catch ex As Exception
            Console.Error.WriteLine($"エラー：{ex.Message}")
            If ex.InnerException IsNot Nothing Then
                Console.Error.WriteLine($"内部例外：{ex.InnerException.Message}")
            End If
        Finally
            If Not yes2all Then
                Console.WriteLine("終了するには何かキーを押下してください...")
                Console.ReadKey()
            End If
            If quiet Then
                outTw.Dispose()
                outMs.Dispose()
            End If
        End Try
    End Sub
End Module
